"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const WebSocket = require("ws");
const child_process = require("child_process");
const vscode_languageclient_1 = require("vscode-languageclient");
const path_1 = require("path");
const os_1 = require("os");
let client;
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => setTimeout(resolve, ms));
    });
}
function activate(context) {
    const execShellCommand = (cmd) => new Promise((resolve, reject) => {
        child_process.exec(cmd, (err, out) => {
            if (err)
                return reject(err);
            return resolve(out);
        });
    });
    var ideVersion = undefined;
    execShellCommand(getCommand('ide.getVersionCommand')).then(version => {
        ideVersion = encodeURIComponent(version.trim().split(' ').at(-1));
    }).catch((err) => { });
    const https = require('https');
    var lastPhoneHome = "";
    function phoneHome() {
        if (ideVersion) {
            // ide found, so Verum Dezyne is installed, the non-open source extensions of Dezyne.
            // Send every day, ide version, and user email to verum.com if provided, otherwise ask to do so.
            let user = encodeURIComponent(vscode.workspace.getConfiguration('dezyne').get('ide.userEmail'));
            if (user) {
                var currentDate = (new Date()).toDateString();
                if (lastPhoneHome != currentDate) {
                    lastPhoneHome = currentDate;
                    https.get(`https://download.verum.com/identify/vscode/?ide=${ideVersion}&user=${user}`, () => { });
                }
            }
            else {
                vscode.commands.executeCommand('workbench.action.openSettings', '@ext:verum.dezyne ide.user');
                vscode.window.showErrorMessage("Please provide as a setting the user email address to be send to verum.com at start-up " +
                    "when having the non open source Verum Dezyne extensions installed.", { modal: true });
            }
        }
    }
    function getPath() {
        if (!vscode.window.activeTextEditor)
            return "";
        const { document } = vscode.window.activeTextEditor;
        let path = document.uri.path;
        if ((0, os_1.platform)() === 'win32' && path[0] == "/") {
            path = path.substring(1);
        }
        return path;
    }
    function resolveVariableReferences(command, model) {
        let config = vscode.workspace.getConfiguration('dezyne');
        return command.
            replace(/\${editorPort}/g, config.get('daemon.editorPort')).
            replace(/\${httpPort}/g, config.get('daemon.httpPort')).
            replace(/\${idePort}/g, config.get('daemon.idePort')).
            replace(/\${viewPort}/g, config.get('daemon.viewPort')).
            replace(/\${importPath}/g, config.get('ide.importPath')).
            replace(/\${codeOutputPath}/g, config.get('dzn.codeOutputPath')).
            replace(/\${file}/g, getPath()).
            replace(/\${model}/g, model);
    }
    function getTerminal(name, isTransient = false) {
        let cwd = (vscode.workspace.workspaceFolders && vscode.workspace.workspaceFolders[0].uri) || (0, path_1.dirname)(getPath());
        let terminal = vscode.window.terminals.find(t => t.name == name);
        if (terminal && terminal.creationOptions.cwd != cwd) {
            terminal.dispose();
            terminal = null;
        }
        if (!terminal) {
            terminal = vscode.window.createTerminal({ name: name, cwd: cwd, isTransient: isTransient });
        }
        return terminal;
    }
    var prev_command = null;
    var prev_terminalName = null;
    function executeCommand(command, terminalName) {
        phoneHome();
        prev_command = command;
        prev_terminalName = terminalName;
        if (vscode.workspace.getConfiguration('dezyne').get('ide.autoSave')) {
            vscode.commands.executeCommand("workbench.action.files.saveFiles");
        }
        let terminal = getTerminal(terminalName);
        terminal.show(true);
        terminal.sendText(command, true);
    }
    function repeatCommand() {
        if (prev_command) {
            executeCommand(prev_command, prev_terminalName);
        }
    }
    function getCommandArgs(property, model) {
        const path = vscode.workspace.getConfiguration('dezyne').get('path');
        let command = vscode.workspace.getConfiguration('dezyne').get(property);
        command = resolveVariableReferences(command, model);
        let args = command.split(' ').filter(str => str.length != 0);
        command = args[0];
        args = args.slice(1);
        if ((0, os_1.platform)() === 'win32' && !command.includes('.')) {
            command += '.cmd';
        }
        if (path !== '' && !(0, path_1.isAbsolute)(command)) {
            command = path + path_1.sep + command;
        }
        return { command: command, args: args };
    }
    function getCommand(property, model) {
        let cmd = getCommandArgs(property, model);
        return cmd.command + ' ' + cmd.args.join(' ');
    }
    const serverCommand = getCommandArgs('server.command');
    const serverOptions = {
        command: serverCommand.command,
        args: serverCommand.args
    };
    const clientOptions = {
        documentSelector: [{ scheme: 'file', language: 'dzn' }],
    };
    const client = new vscode_languageclient_1.LanguageClient('dezyne', 'Dezyne Language Server', serverOptions, clientOptions);
    context.subscriptions.push(client.start());
    const daemonTerminalName = 'Dezyne Daemon';
    if (vscode.workspace.getConfiguration('dezyne').get('daemon.autoStart')) {
        const terminal = getTerminal(daemonTerminalName, true);
        terminal.sendText(getCommand('daemon.commandStop'), true);
        terminal.sendText(getCommand('daemon.commandStart'), true);
    }
    function selectModel(placeHolder, filterType, error) {
        return execShellCommand(getCommand('dzn.listModelsCommand')).then(list => {
            let models = list.split(/\r?\n/).filter((entry) => filterType.includes(entry.split(' ')[1]));
            models = models.map(entry => entry.split(' ')[0]);
            if (models.length == 0) {
                vscode.window.showErrorMessage(error, { modal: true });
                return undefined;
            }
            if (models.length == 1)
                return Promise.resolve(models[0]);
            return vscode.window.showQuickPick(models, { placeHolder: placeHolder });
        }).catch((err) => {
            vscode.window.showErrorMessage(`ERROR:\n${err.message}`, { modal: true });
            return undefined;
        });
    }
    const filterHasBehavior = ['interface', 'component'];
    const filterNotForeign = ['interface', 'component', 'system'];
    const filterSystem = ['system'];
    const showDaemonTerminalHandler = () => {
        getTerminal(daemonTerminalName).show();
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.showDaemonTerminal', showDaemonTerminalHandler));
    const simulateHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        selectModel('Pick model to be simulated.', filterNotForeign, 'No model with behavior found.').then(model => {
            if (!model)
                return;
            executeCommand(getCommand('ide.simulateCommand', model), "Dezyne Simulate");
        });
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.simulate', simulateHandler));
    const stateDiagramHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        selectModel('Pick model for State Diagram.', filterNotForeign, 'No model with behavior found.').then(model => {
            if (!model)
                return;
            executeCommand(getCommand('ide.stateDiagramCommand', model), "Dezyne State Diagram View");
        });
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.stateDiagram', stateDiagramHandler));
    const systemHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        selectModel('Pick model for System View.', filterSystem, 'No system model found.').then(model => {
            if (!model)
                return;
            executeCommand(getCommand('ide.systemCommand', model), 'Dezyne System View');
        });
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.system', systemHandler));
    const parseHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        executeCommand(getCommand('dzn.parseCommand'), 'Dezyne Parse');
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.parse', parseHandler));
    const verifyHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        selectModel('Pick model to be verified.', filterHasBehavior, 'No model with behavior found.').then(model => {
            if (!model)
                return;
            executeCommand(getCommand('ide.verifyCommand', model), 'Dezyne Verify');
        });
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.verify', verifyHandler));
    const verifyFileHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        executeCommand(getCommand('ide.verifyFileCommand'), 'Dezyne Verify File');
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.verifyFile', verifyFileHandler));
    const codeHandler = () => {
        if (!vscode.window.activeTextEditor)
            return;
        if (vscode.workspace.getConfiguration('dezyne').get('dzn.codeOutputPath') != "") {
            executeCommand(getCommand('dzn.codeCommand'), 'Dezyne Code');
        }
        else {
            vscode.commands.executeCommand('workbench.action.openWorkspaceSettings');
            vscode.commands.executeCommand('workbench.action.openSettings', '@ext:verum.dezyne dzn.code');
            vscode.window.showErrorMessage("Dezyne Code Command cannot be executed due to missing Code Output Path setting.\n\n" +
                "Please provide as a Workspace setting the Code Output Path for writing the generated code, " +
                "and rerun Dezyne Code Command.", { modal: true });
        }
    };
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.code', codeHandler));
    context.subscriptions.push(vscode.commands.registerCommand('dezyne.repeatCommand', repeatCommand));
    let dznButtons = [];
    function createButton(command, tooltip, icon, pos) {
        let button = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left, pos);
        button.command = command;
        button.tooltip = tooltip;
        button.text = icon;
        button.show();
        dznButtons.push(button);
        context.subscriptions.push(button);
        return button;
    }
    context.subscriptions.push(vscode.window.onDidChangeActiveTextEditor(function (editor) {
        const show = editor && (editor.document.languageId == 'dzn');
        if (show)
            phoneHome();
        dznButtons.forEach((btn) => { if (show)
            btn.show();
        else
            btn.hide(); });
    }));
    const daemonButton = createButton('dezyne.showDaemonTerminal', '', '', 6);
    const parseButton = createButton('dezyne.parse', 'Report Well-formedness Errors', '$(inspect)', 5);
    const stateDiagramButton = createButton('dezyne.stateDiagram', 'Show State Diagram...', '$(circuit-board)', 4);
    const systemButton = createButton('dezyne.system', 'Show System View', '$(preview)', 3);
    const simulateButton = createButton('dezyne.simulate', 'Simulate Model...', '$(run)', 2);
    const verifyButton = createButton('dezyne.verify', 'Verify Model...', '$(run-all)', 1);
    const repeatButton = createButton('dezyne.repeatCommand', 'Repeat Previous Command', '$(debug-restart)', 0);
    function setDaemonConnectedStatus(open) {
        daemonButton.tooltip = open ? 'Connected to Dezyne daemon.' : 'Trying to connect to Dezyne daemon...';
        daemonButton.text = open ? '$(pass)' : '$(sync~spin)';
        daemonButton.color = open ? undefined : new vscode.ThemeColor('problemsErrorIcon.foreground');
    }
    const gotoDecoration = vscode.window.createTextEditorDecorationType({ backgroundColor: { id: 'editor.selectionBackground' } });
    let gotoDecorationSelection = null;
    let gotoDecorationEditor = null;
    context.subscriptions.push(vscode.window.onDidChangeTextEditorSelection(function (event) {
        if (gotoDecorationSelection && !event.selections[0].isEqual(gotoDecorationSelection)) {
            gotoDecorationEditor.setDecorations(gotoDecoration, []);
            gotoDecorationSelection = null;
        }
    }));
    function connectDaemon(connection) {
        connection.status = 'closed';
        setDaemonConnectedStatus(false);
        function reconnectDaemon(connection) {
            if (connection.status == 'open' && !connection.silent) {
                setDaemonConnectedStatus(false);
            }
            if (connection.status != 'waiting') {
                connection.status = 'waiting';
                setTimeout(function () { connectDaemon(connection); }, 1000);
            }
        }
        try {
            connection.status = 'connecting';
            let socket = new WebSocket(`ws://127.0.0.1:${connection.port}`);
            socket.onopen = function () {
                connection.status = 'open';
                if (!connection.silent) {
                    setDaemonConnectedStatus(true);
                }
            };
            socket.onerror = function (ev) { reconnectDaemon(connection); };
            socket.onclose = function (ev) { reconnectDaemon(connection); };
            socket.onmessage = connection.onmessage;
            connection.socket = socket;
        }
        catch (error) {
            reconnectDaemon(connection);
        }
    }
    function setDecoration(editor, range) {
        editor.setDecorations(gotoDecoration, [range]);
        gotoDecorationEditor = editor;
        gotoDecorationSelection = new vscode.Selection(range.start, range.start);
        editor.selection = gotoDecorationSelection;
        editor.revealRange(range, vscode.TextEditorRevealType.InCenterIfOutsideViewport);
    }
    function onGotoMessage(ev) {
        const loc = JSON.parse(ev.data.toString())['go_to'];
        let working_directory = loc['working-directory'];
        let file_name = loc['file-name'];
        if (!((0, path_1.isAbsolute)(file_name)) && working_directory) {
            file_name = working_directory + path_1.sep + file_name;
        }
        vscode.workspace.openTextDocument(file_name).then(document => {
            vscode.window.showTextDocument(document, vscode.ViewColumn.One, true).then(editor => {
                const pos0 = new vscode.Position(loc['line'] - 1, loc['column'] - 1);
                const endline = loc['end-line'] || loc['line'];
                const endcolumn = loc['end-column'] || loc['column'];
                var pos1 = new vscode.Position(endline - 1, endcolumn - 1);
                var range;
                if (pos0.isEqual(pos1)) {
                    range = editor.document.getWordRangeAtPosition(pos0, /(on\s)?[^:;{}\s]+/);
                    if (!range) {
                        pos1 = pos0.translate(0, 1);
                    }
                }
                if (!range) {
                    range = new vscode.Range(pos0, pos1);
                }
                setDecoration(editor, range);
            });
        });
    }
    if (vscode.workspace.getConfiguration('dezyne').get('daemon.gotoClick')) {
        const port = vscode.workspace.getConfiguration('dezyne').get('daemon.editorPort');
        connectDaemon({ port: port, onmessage: onGotoMessage, silent: false });
    }
    if (vscode.workspace.getConfiguration('dezyne').get('ide.webviews')) {
        const port = vscode.workspace.getConfiguration('dezyne').get('daemon.viewPort');
        const silent = vscode.workspace.getConfiguration('dezyne').get('daemon.gotoClick');
        connectDaemon({ port: port, onmessage: onViewMessage, silent: silent });
    }
    function createDznView(title, page) {
        function getWebviewContent() {
            const port = vscode.workspace.getConfiguration('dezyne').get('daemon.httpPort');
            return `<!DOCTYPE html>
			<html lang="en">
			<head>
			  <meta charset="UTF-8">
			  <meta name="viewport" content="width=device-width, initial-scale=1.0">
			  <title>Dezyne-IDE Views</title>
			</head><body>
			<style> body,html{ height: 100%; margin:0; padding:0; overflow:hidden; } </style>
			<iframe id="${page}_iframe" class="view" src="http://localhost:${port}/${page}.html"
			  style="position:relative; border:0; height:100%; width:100%;"
			  onload="this.contentWindow.focus()"></iframe>
			<script>
			window.addEventListener('message', event => {
				console.log(event);
				const message = event.data;
				switch (message.command) {
				case 'focus':
					document.getElementById("${page}_iframe").contentWindow.focus();
					break;
				}
			})
			window.addEventListener('mouseup', event => {
				document.getElementById("${page}_iframe").contentWindow.focus();
			})
			</script>
			</body></html>`;
        }
        let panel = vscode.window.createWebviewPanel('dznview', title, { viewColumn: vscode.ViewColumn.Two, preserveFocus: true }, { enableScripts: true, retainContextWhenHidden: true });
        panel.webview.html = getWebviewContent();
        panel.webview.postMessage({ command: "focus" });
        panel.onDidChangeViewState(e => {
            const panel = e.webviewPanel;
            if (panel.active)
                panel.webview.postMessage({ command: "focus" });
        });
        return panel;
    }
    let views = {
        state: { title: "State Diagram", type: "state", panel: null },
        parse: { title: "System View", type: "system", panel: null },
        simulate: { title: "Sequence Diagram", type: "trace", panel: null }
    };
    function onViewMessage(ev) {
        const msg = JSON.parse(ev.data.toString());
        let notify = msg.notify;
        if (notify) {
            let label = notify.label;
            if (views[label]) {
                if (!views[label].panel) {
                    views[label].panel = createDznView(views[label].title, views[label].type);
                    views[label].panel.onDidDispose((ev) => { views[label].panel = null; });
                }
                else {
                    views[label].panel.reveal(undefined, true);
                }
            }
        }
    }
}
exports.activate = activate;
function deactivate() {
    if (!client) {
        return undefined;
    }
    return client.stop();
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map